//	Initialize the formatting options
prism.run([function() {	

	//	Define allowed chart types
	var supportedChartTypes = ["pivot"];
	
	//	Options
	var checkboxProperty = 'checkbox';
	var none = false;
		
	//	Function to kick off a regression analysis
	function addCheckboxes(widget,args) {		

		//	Get a list of the value items
		var section = "";
		switch(widget.type) {
			case "pivot": section = "rows"; break;		
		}		
		var valueItems = getItems(widget,section);		
		
		if (valueItems) {
			//	Loop through each series
			for (var i=0; i<valueItems.items.length; i++){
				
				//	Should we format the time?
				var savedCheckbox = valueItems.items[i][checkboxProperty] ? valueItems.items[i][checkboxProperty] : none;

				if (savedCheckbox) {

					//	Check for the widget type
					switch(widget.type) {
						case "pivot" : 
							//	Handle the HTML of pivot tables
							args.result = formatPivot(args.result, args.query.metadata, i);
							break;
					}
				}				
			}	
		}
	}

	//////////////////////////////////////
	//	Widget Specific Functions		//
	//////////////////////////////////////	

	//	Function to format the values for a pivot table
	function formatPivot(result, metadata, index){
		
		//	Figure out the fidx		
		var valueItems = $.grep(metadata, function(w){
			return w.panel === "rows";
		})

		//	Figure out THIS item
		var thisItem = valueItems[index];
		var thisFidx = thisItem.field.index;
		var thisCheckboxFormat = thisItem.checkbox;		
		var thisJaql = JSON.stringify({
			column: thisItem.jaql.column,
			table: thisItem.jaql.table,
			dim: thisItem.jaql.dim,
			datatype: thisItem.jaql.datatype,
			title: thisItem.jaql.title
		})

		//	Get the dashboard filter's selections
		var thisFilter = getFilterFromDashboard(thisItem.jaql.dim);
		var thisFilterMembers = [];
		var allItems = false;		
		if (thisFilter) {

			//	Get the filter's jaql
			var jaqlFilter;
			if (thisFilter.isCascading) {
				//	Handle cascading filter
				for (var i=0; i<thisFilter.levels.length; i++) {
					if (thisFilter.levels[i].dim == thisItem.jaql.dim){
						jaqlFilter = thisFilter.levels[i].filter;
					}
				}
			} else {
				//	Normal filter
				jaqlFilter = thisFilter.jaql.filter;
			}

			//	Check the filter's selection	
			if (jaqlFilter.members) {
				thisFilterMembers = jaqlFilter.members;
			} else if (jaqlFilter.all) {
				allItems = true;
			}
			
		} 
		

		//	Load the HTML table
		var table =  $('<div>').append($(result.table));
		var tableBody = $('tbody',table);

		//	Find all cells that match this fidx
		var cells = $('td.p-dim-member[fIdx=' + thisFidx + ']',tableBody);	

		//	Loop through each cell, and format the time
		$.each(cells, function(){			
			//	Get the existing text
			var oldHtml = $(this).text();

			//	Is this members selected already?
			var isChecked = '';
			if (allItems || thisFilterMembers.indexOf(oldHtml)>=0){
				isChecked = 'checked'
			}
			
			//	Create the checkbox
			var newHTML = "<input type='checkbox' filterJaql='" + thisJaql + "' onClick='prism.setCheckboxFilter(this)' " + isChecked + ">" + oldHtml;

			//	Save the new time			
			$(this).html(newHTML);
		})
		
		//	Save the updated HTML table back to the result
		result.table = table.html();		

		//	Pass back the updated result
		return result;
	}

	//	Function to set the filter
	prism.setCheckboxFilter = function(element) {
		
		//	Parse the filter object and selection
		var filterJaql = JSON.parse($(element).attr('filterJaql'));
		var newSelection = $(element).parent().text();

		//	Get the dashboard filter
		var thisFilter = getFilterFromDashboard(filterJaql.dim);

		//	Create the jaql properties
		var dim, title, table, column, datatype, members = [];

		//	Create the base of the filter object
		var filterObj = {
		};

		//	Filter already exists
		if (thisFilter) {

			
			if (thisFilter.isCascading) {
				//	Cascading filter
				
				filterObj.isCascading = true;
				filterObj.levels = []
				for (var i=0;i<thisFilter.levels.length; i++){
					//	Copy the level
					var levelCopy = $.extend({},thisFilter.levels[i]);
					if (levelCopy.dim == filterJaql.dim) {
						//	Are there filter members already?
						if (levelCopy.filter.members) {
							//	Check to see if the member exists already
							var selectedMemberIndex = levelCopy.filter.members.indexOf(newSelection);
							if (selectedMemberIndex>=0) {
								//	Cut it out of the filter to de-select
								levelCopy.filter.members.splice(selectedMemberIndex,1);
							} else {
								//	Add it to the list
								levelCopy.filter.members.push(newSelection)
							}
						} else {
							//	New member list
							levelCopy.filter.members = [newSelection];
						}
						
					}

					filterObj.levels.push(levelCopy);
				}
			} else {
				//	Regular filter, 
				filterObj.jaql = {};
				filterObj.jaql.dim = thisFilter.jaql.dim;
				filterObj.jaql.title = thisFilter.jaql.title;
				filterObj.jaql.table = thisFilter.jaql.table;
				filterObj.jaql.column = thisFilter.jaql.column;
				filterObj.jaql.datatype = thisFilter.jaql.datatype;
				filterObj.jaql.filter = {
					members: thisFilter.jaql.filter.members ? thisFilter.jaql.filter.members : []
				}

				//	Check if the new text is already selected
				members = filterObj.jaql.filter.members;
				var selectedMemberIndex = members.indexOf(newSelection);
				if (selectedMemberIndex>=0) {
					//	Cut it out of the filter to de-select
					members.splice(selectedMemberIndex,1);
				} else {
					//	Add it to the list
					members.push(newSelection)
				}
			}
			
		} else {

			//	Set properties for a new filter
			filterObj.jaql = {};
			filterObj.jaql.dim = filterJaql.dim;
			filterObj.jaql.title = filterJaql.title;
			filterObj.jaql.table = filterJaql.table;
			filterObj.jaql.column = filterJaql.column;
			filterObj.jaql.datatype = filterJaql.datatype;
			filterObj.jaql.filter = {
				members: [newSelection]
			};
		}

		//	Define the options
		var filterOptions = {
			save:true,
			refresh:true,
			unionIfSameDimensionAndSameType:true
		}

		//	Set the filter
		prism.activeDashboard.filters.update(filterObj, filterOptions);
	}
	
	//////////////////////////////////
	//	Utility Functions			//
	//////////////////////////////////

	//	function to find the value items from a widget
	function getItems(widget,section) {
		var valueItems = $.grep(widget.metadata.panels, function(w){
			return w.title===section;
		})[0];
		return valueItems;
	}

	//	Function to get a dashboard filter with matching dim
	function getFilterFromDashboard(dim) {
		//	Get the dashboard filters
		var filters = prism.activeDashboard.filters.$$items;
		
		//	Search for a filter with the same dim
		var thisFilter = $.grep(filters,function(w){			
			if (w.isCascading) {
				//	Handle cascading filters
				var isMatch = false;
				for (var i=0; i<w.levels.length; i++){
					if (w.levels[i].dim == dim) {
						isMatch = true;
					}
				};
				return isMatch;
			} else {
				//	Normal filter
				return w.jaql.dim === dim;	
			}			
		})[0];

		//	Return the result
		return thisFilter;
	}

	//	Function to determine if this chart type is supported
	function widgetIsSupported(type){
		if (supportedChartTypes.indexOf(type) >= 0) {
			return true;
		} else {
			return false;
		}
	}

	//	Function to determine if the checkbox option should be added
	function canEnabledCheckboxes(e,args,headerMenuCaption,menuType){
		
		//	Assume true by default
		var result = true;				
		try {			
			//	Has the menu been added already?			
			$.each(args.settings.items, function(){
				if (this.caption === headerMenuCaption) {
					result = false;
				}
			})

			//	Only show this in the widget editor
			var widgetEditorOpen = ($('.tab-design').length > 0);
			if (!widgetEditorOpen) {
				result = false;
			}
			
			// find the widget
			var widget = e.currentScope.widget;
			if(typeof widget === "undefined"){
				result = false;
			}

			// Widget must be an allowed chart
			if(!widgetIsSupported(widget.type)){
				result = false;
			}

			//	Check where the user clicked
			var target = args.ui.target;
			//	Make sure the user clicked on a measures menu
			if (args.settings.name !== "widget-metadataitem") {
				result = false;
			}
			//	For pivot charts, make sure the settings menu is for a value
			if (e.currentScope.widgetType === "pivot") {
				if (args.settings.item.$$panel.name !== "rows") {
					result = false;
				}
			}
						
		} catch(e) {
			result = false;
		}

		//	Return result
		return result;
	}

	//////////////////////////////////
	//	Initialization Functions	//
	//////////////////////////////////

	// Registering dashboard/ widget creation in order to perform drilling
	prism.on("dashboardloaded", function (e, args) {
		args.dashboard.on("widgetinitialized", onWidgetInitialized);
	});

	// register widget events upon initialization
	function onWidgetInitialized(dashboard, args) {
		//	Hooking to ready/destroyed events
		args.widget.on("destroyed", onWidgetDestroyed);		
		//	Add hook for the time formatting
		var shouldInit = widgetIsSupported(args.widget.type);
		if (shouldInit) {
			args.widget.on("processresult", addCheckboxes);
		}
	}

	// unregistering widget events
	function onWidgetDestroyed(widget, args) {
		widget.off("destroyed", onWidgetDestroyed);
	}

	//	Create Options for the quadrant analysis
	prism.on("beforemenu", function (e, args) {

		//	Define the menu label
		var headerMenuCaption = "Checkboxes";

		//	Can we show the regression options?	
		var addCheckboxMenuItem = canEnabledCheckboxes(e,args,headerMenuCaption);				

		//	Add the option to enable the regression line
		if (addCheckboxMenuItem) {			
			
			//	Get the widget
			var widget = args.settings.item.$$panel.$$widget;	

			//	Look for a saved regression property		
			var savedCheckbox = args.settings.item[checkboxProperty] ? args.settings.item[checkboxProperty] : none;

			//	Function the runs when an item is picked
			var itemPicked = function(){						
				//	toggle the setting
				args.settings.item[checkboxProperty] = !savedCheckbox;

				//	Set some default widget settings automatically		
				/*		
				if (!savedCheckbox) {
					widget.options.dashboardFiltersMode = "select";
					widget.options.drillToAnywhere = false;
					widget.options.selector = false;
				}
				*/

				//	Redraw the widget
				widget.redraw();						
			}

			//	Create menu for picking the regression measure
			var separator = {
				type: 'separator'
			};
			var checkboxItem = {
				caption: headerMenuCaption,
				checked: (savedCheckbox === true),
				size: 'xl',
				type: 'check',
				execute: itemPicked
			};

			//	Add options to the menu			
			args.settings.items.push(separator);	
			args.settings.items.push(checkboxItem);	
		}
		
	});

}]);